@@ -1,54 +1,30 @@
-import operator
-
+"""
+This module tests the functionality of StringArray and ArrowStringArray.
+Tests for the str accessors are in pandas/tests/strings/test_string_array.py
+"""
 import numpy as np
 import pytest
 
 import pandas.util._test_decorators as td
 
+from pandas.core.dtypes.common import is_dtype_equal
+
 import pandas as pd
 import pandas._testing as tm
-from pandas.core.arrays.string_arrow import ArrowStringArray, ArrowStringDtype
-
-skip_if_no_pyarrow = td.skip_if_no("pyarrow", min_version="1.0.0")
-
-
-@pytest.fixture(
-    params=[
-        "string",
-        pytest.param("arrow_string", marks=skip_if_no_pyarrow),
-    ]
-)
-def dtype(request):
-    return request.param
+from pandas.core.arrays.string_arrow import ArrowStringArray
 
 
 @pytest.fixture
-def dtype_object(dtype):
-    if dtype == "string":
-        return pd.StringDtype
-    else:
-        return ArrowStringDtype
+def dtype(string_storage):
+    return pd.StringDtype(storage=string_storage)
 
 
-@pytest.fixture(
-    params=[
-        pd.arrays.StringArray,
-        pytest.param(ArrowStringArray, marks=skip_if_no_pyarrow),
-    ]
-)
-def cls(request):
-    return request.param
+@pytest.fixture
+def cls(dtype):
+    return dtype.construct_array_type()
 
 
-def test_repr(dtype, request):
-    if dtype == "arrow_string":
-        reason = (
-            "AssertionError: assert '      A\n0     a\n1  None\n2     b' "
-            "== '      A\n0     a\n1  <NA>\n2     b'"
-        )
-        mark = pytest.mark.xfail(reason=reason)
-        request.node.add_marker(mark)
-
+def test_repr(dtype):
     df = pd.DataFrame({"A": pd.array(["a", pd.NA, "b"], dtype=dtype)})
     expected = "      A\n0     a\n1  <NA>\n2     b"
     assert repr(df) == expected
@@ -56,7 +32,8 @@ def test_repr(dtype, request):
     expected = "0       a\n1    <NA>\n2       b\nName: A, dtype: string"
     assert repr(df.A) == expected
 
-    expected = "<StringArray>\n['a', <NA>, 'b']\nLength: 3, dtype: string"
+    arr_name = "ArrowStringArray" if dtype.storage == "pyarrow" else "StringArray"
+    expected = f"<{arr_name}>\n['a', <NA>, 'b']\nLength: 3, dtype: string"
     assert repr(df.A.array) == expected
 
 
@@ -93,49 +70,34 @@ def test_setitem_with_scalar_string(dtype):
     tm.assert_extension_array_equal(arr, expected)
 
 
-@pytest.mark.parametrize(
-    "input, method",
-    [
-        (["a", "b", "c"], operator.methodcaller("capitalize")),
-        (["a", "b", "c"], operator.methodcaller("capitalize")),
-        (["a b", "a bc. de"], operator.methodcaller("capitalize")),
-    ],
-)
-def test_string_methods(input, method, dtype, request):
-    if dtype == "arrow_string":
-        reason = "AttributeError: 'ArrowStringDtype' object has no attribute 'base'"
-        mark = pytest.mark.xfail(reason=reason)
-        request.node.add_marker(mark)
-
-    a = pd.Series(input, dtype=dtype)
-    b = pd.Series(input, dtype="object")
-    result = method(a.str)
-    expected = method(b.str)
-
-    assert result.dtype.name == dtype
-    tm.assert_series_equal(result.astype(object), expected)
-
-
 def test_astype_roundtrip(dtype, request):
-    if dtype == "arrow_string":
+    if dtype.storage == "pyarrow":
         reason = "ValueError: Could not convert object to NumPy datetime"
-        mark = pytest.mark.xfail(reason=reason)
+        mark = pytest.mark.xfail(reason=reason, raises=ValueError)
+        request.node.add_marker(mark)
+    else:
+        mark = pytest.mark.xfail(
+            reason="GH#36153 casting from StringArray to dt64 fails", raises=ValueError
+        )
         request.node.add_marker(mark)
 
-    s = pd.Series(pd.date_range("2000", periods=12))
-    s[0] = None
+    ser = pd.Series(pd.date_range("2000", periods=12))
+    ser[0] = None
 
-    result = s.astype(dtype).astype("datetime64[ns]")
-    tm.assert_series_equal(result, s)
+    casted = ser.astype(dtype)
+    assert is_dtype_equal(casted.dtype, dtype)
+
+    result = casted.astype("datetime64[ns]")
+    tm.assert_series_equal(result, ser)
 
 
 def test_add(dtype, request):
-    if dtype == "arrow_string":
+    if dtype.storage == "pyarrow":
         reason = (
-            "TypeError: unsupported operand type(s) for +: 'ArrowStringArray' and "
+            "unsupported operand type(s) for +: 'ArrowStringArray' and "
             "'ArrowStringArray'"
         )
-        mark = pytest.mark.xfail(reason=reason)
+        mark = pytest.mark.xfail(raises=TypeError, reason=reason)
         request.node.add_marker(mark)
 
     a = pd.Series(["a", "b", "c", None, None], dtype=dtype)
@@ -158,9 +120,9 @@ def test_add(dtype, request):
 
 
 def test_add_2d(dtype, request):
-    if dtype == "arrow_string":
+    if dtype.storage == "pyarrow":
         reason = "Failed: DID NOT RAISE <class 'ValueError'>"
-        mark = pytest.mark.xfail(reason=reason)
+        mark = pytest.mark.xfail(raises=None, reason=reason)
         request.node.add_marker(mark)
 
     a = pd.array(["a", "b", "c"], dtype=dtype)
@@ -174,12 +136,9 @@ def test_add_2d(dtype, request):
 
 
 def test_add_sequence(dtype, request):
-    if dtype == "arrow_string":
-        reason = (
-            "TypeError: unsupported operand type(s) for +: 'ArrowStringArray' "
-            "and 'list'"
-        )
-        mark = pytest.mark.xfail(reason=reason)
+    if dtype.storage == "pyarrow":
+        reason = "unsupported operand type(s) for +: 'ArrowStringArray' and 'list'"
+        mark = pytest.mark.xfail(raises=TypeError, reason=reason)
         request.node.add_marker(mark)
 
     a = pd.array(["a", "b", None, None], dtype=dtype)
@@ -195,11 +154,9 @@ def test_add_sequence(dtype, request):
 
 
 def test_mul(dtype, request):
-    if dtype == "arrow_string":
-        reason = (
-            "TypeError: unsupported operand type(s) for *: 'ArrowStringArray' and 'int'"
-        )
-        mark = pytest.mark.xfail(reason=reason)
+    if dtype.storage == "pyarrow":
+        reason = "unsupported operand type(s) for *: 'ArrowStringArray' and 'int'"
+        mark = pytest.mark.xfail(raises=TypeError, reason=reason)
         request.node.add_marker(mark)
 
     a = pd.array(["a", "b", None], dtype=dtype)
@@ -213,37 +170,37 @@ def test_mul(dtype, request):
 
 @pytest.mark.xfail(reason="GH-28527")
 def test_add_strings(dtype):
-    array = pd.array(["a", "b", "c", "d"], dtype=dtype)
+    arr = pd.array(["a", "b", "c", "d"], dtype=dtype)
     df = pd.DataFrame([["t", "u", "v", "w"]])
-    assert array.__add__(df) is NotImplemented
+    assert arr.__add__(df) is NotImplemented
 
-    result = array + df
+    result = arr + df
     expected = pd.DataFrame([["at", "bu", "cv", "dw"]]).astype(dtype)
     tm.assert_frame_equal(result, expected)
 
-    result = df + array
+    result = df + arr
     expected = pd.DataFrame([["ta", "ub", "vc", "wd"]]).astype(dtype)
     tm.assert_frame_equal(result, expected)
 
 
 @pytest.mark.xfail(reason="GH-28527")
 def test_add_frame(dtype):
-    array = pd.array(["a", "b", np.nan, np.nan], dtype=dtype)
+    arr = pd.array(["a", "b", np.nan, np.nan], dtype=dtype)
     df = pd.DataFrame([["x", np.nan, "y", np.nan]])
 
-    assert array.__add__(df) is NotImplemented
+    assert arr.__add__(df) is NotImplemented
 
-    result = array + df
+    result = arr + df
     expected = pd.DataFrame([["ax", np.nan, np.nan, np.nan]]).astype(dtype)
     tm.assert_frame_equal(result, expected)
 
-    result = df + array
+    result = df + arr
     expected = pd.DataFrame([["xa", np.nan, np.nan, np.nan]]).astype(dtype)
     tm.assert_frame_equal(result, expected)
 
 
-def test_comparison_methods_scalar(all_compare_operators, dtype):
-    op_name = all_compare_operators
+def test_comparison_methods_scalar(comparison_op, dtype):
+    op_name = f"__{comparison_op.__name__}__"
     a = pd.array(["a", None, "c"], dtype=dtype)
     other = "a"
     result = getattr(a, op_name)(other)
@@ -252,23 +209,26 @@ def test_comparison_methods_scalar(all_compare_operators, dtype):
     tm.assert_extension_array_equal(result, expected)
 
 
-def test_comparison_methods_scalar_pd_na(all_compare_operators, dtype):
-    op_name = all_compare_operators
+def test_comparison_methods_scalar_pd_na(comparison_op, dtype):
+    op_name = f"__{comparison_op.__name__}__"
     a = pd.array(["a", None, "c"], dtype=dtype)
     result = getattr(a, op_name)(pd.NA)
     expected = pd.array([None, None, None], dtype="boolean")
     tm.assert_extension_array_equal(result, expected)
 
 
-def test_comparison_methods_scalar_not_string(all_compare_operators, dtype, request):
-    if all_compare_operators not in ["__eq__", "__ne__"]:
-        reason = "comparison op not supported between instances of 'str' and 'int'"
-        mark = pytest.mark.xfail(reason=reason)
-        request.node.add_marker(mark)
+def test_comparison_methods_scalar_not_string(comparison_op, dtype):
+    op_name = f"__{comparison_op.__name__}__"
 
-    op_name = all_compare_operators
     a = pd.array(["a", None, "c"], dtype=dtype)
     other = 42
+
+    if op_name not in ["__eq__", "__ne__"]:
+        with pytest.raises(TypeError, match="not supported between"):
+            getattr(a, op_name)(other)
+
+        return
+
     result = getattr(a, op_name)(other)
     expected_data = {"__eq__": [False, None, False], "__ne__": [True, None, True]}[
         op_name
@@ -277,16 +237,9 @@ def test_comparison_methods_scalar_not_string(all_compare_operators, dtype, requ
     tm.assert_extension_array_equal(result, expected)
 
 
-def test_comparison_methods_array(all_compare_operators, dtype, request):
-    if dtype == "arrow_string":
-        if all_compare_operators in ["__eq__", "__ne__"]:
-            reason = "NotImplementedError: Neither scalar nor ArrowStringArray"
-        else:
-            reason = "AssertionError: left is not an ExtensionArray"
-        mark = pytest.mark.xfail(reason=reason)
-        request.node.add_marker(mark)
+def test_comparison_methods_array(comparison_op, dtype):
 
-    op_name = all_compare_operators
+    op_name = f"__{comparison_op.__name__}__"
 
     a = pd.array(["a", None, "c"], dtype=dtype)
     other = [None, None, "c"]
@@ -326,8 +279,9 @@ def test_constructor_raises(cls):
 @pytest.mark.parametrize("copy", [True, False])
 def test_from_sequence_no_mutate(copy, cls, request):
     if cls is ArrowStringArray and copy is False:
-        reason = "AssertionError: numpy array are different"
-        mark = pytest.mark.xfail(reason=reason)
+        mark = pytest.mark.xfail(
+            raises=AssertionError, reason="numpy array are different"
+        )
         request.node.add_marker(mark)
 
     nan_arr = np.array(["a", np.nan], dtype=object)
@@ -348,12 +302,19 @@ def test_from_sequence_no_mutate(copy, cls, request):
     tm.assert_numpy_array_equal(nan_arr, expected)
 
 
-def test_astype_int(dtype, request):
-    if dtype == "arrow_string":
-        reason = "TypeError: Cannot interpret 'Int64Dtype()' as a data type"
-        mark = pytest.mark.xfail(reason=reason)
-        request.node.add_marker(mark)
+def test_astype_int(dtype):
+    arr = pd.array(["1", "2", "3"], dtype=dtype)
+    result = arr.astype("int64")
+    expected = np.array([1, 2, 3], dtype="int64")
+    tm.assert_numpy_array_equal(result, expected)
 
+    arr = pd.array(["1", pd.NA, "3"], dtype=dtype)
+    msg = r"int\(\) argument must be a string, a bytes-like object or a( real)? number"
+    with pytest.raises(TypeError, match=msg):
+        arr.astype("int64")
+
+
+def test_astype_nullable_int(dtype):
     arr = pd.array(["1", pd.NA, "3"], dtype=dtype)
 
     result = arr.astype("Int64")
@@ -361,12 +322,11 @@ def test_astype_int(dtype, request):
     tm.assert_extension_array_equal(result, expected)
 
 
-def test_astype_float(any_float_allowed_nullable_dtype):
+def test_astype_float(dtype, any_float_dtype):
     # Don't compare arrays (37974)
-    ser = pd.Series(["1.1", pd.NA, "3.3"], dtype="string")
-
-    result = ser.astype(any_float_allowed_nullable_dtype)
-    expected = pd.Series([1.1, np.nan, 3.3], dtype=any_float_allowed_nullable_dtype)
+    ser = pd.Series(["1.1", pd.NA, "3.3"], dtype=dtype)
+    result = ser.astype(any_float_dtype)
+    expected = pd.Series([1.1, np.nan, 3.3], dtype=any_float_dtype)
     tm.assert_series_equal(result, expected)
 
 
@@ -378,12 +338,23 @@ def test_reduce(skipna, dtype):
     assert result == "abc"
 
 
+@pytest.mark.parametrize("skipna", [True, False])
+@pytest.mark.xfail(reason="Not implemented StringArray.sum")
+def test_reduce_missing(skipna, dtype):
+    arr = pd.Series([None, "a", None, "b", "c", None], dtype=dtype)
+    result = arr.sum(skipna=skipna)
+    if skipna:
+        assert result == "abc"
+    else:
+        assert pd.isna(result)
+
+
 @pytest.mark.parametrize("method", ["min", "max"])
 @pytest.mark.parametrize("skipna", [True, False])
 def test_min_max(method, skipna, dtype, request):
-    if dtype == "arrow_string":
-        reason = "AttributeError: 'ArrowStringArray' object has no attribute 'max'"
-        mark = pytest.mark.xfail(reason=reason)
+    if dtype.storage == "pyarrow":
+        reason = "'ArrowStringArray' object has no attribute 'max'"
+        mark = pytest.mark.xfail(raises=TypeError, reason=reason)
         request.node.add_marker(mark)
 
     arr = pd.Series(["a", "b", "c", None], dtype=dtype)
@@ -398,15 +369,12 @@ def test_min_max(method, skipna, dtype, request):
 @pytest.mark.parametrize("method", ["min", "max"])
 @pytest.mark.parametrize("box", [pd.Series, pd.array])
 def test_min_max_numpy(method, box, dtype, request):
-    if dtype == "arrow_string":
+    if dtype.storage == "pyarrow":
         if box is pd.array:
-            reason = (
-                "TypeError: '<=' not supported between instances of 'str' and "
-                "'NoneType'"
-            )
+            reason = "'<=' not supported between instances of 'str' and 'NoneType'"
         else:
-            reason = "AttributeError: 'ArrowStringArray' object has no attribute 'max'"
-        mark = pytest.mark.xfail(reason=reason)
+            reason = "'ArrowStringArray' object has no attribute 'max'"
+        mark = pytest.mark.xfail(raises=TypeError, reason=reason)
         request.node.add_marker(mark)
 
     arr = box(["a", "b", "c", None], dtype=dtype)
@@ -415,28 +383,25 @@ def test_min_max_numpy(method, box, dtype, request):
     assert result == expected
 
 
-@pytest.mark.parametrize("skipna", [True, False])
-@pytest.mark.xfail(reason="Not implemented StringArray.sum")
-def test_reduce_missing(skipna, dtype):
-    arr = pd.Series([None, "a", None, "b", "c", None], dtype=dtype)
-    result = arr.sum(skipna=skipna)
-    if skipna:
-        assert result == "abc"
-    else:
-        assert pd.isna(result)
-
-
-def test_fillna_args():
+def test_fillna_args(dtype, request):
     # GH 37987
 
-    arr = pd.array(["a", pd.NA], dtype="string")
+    if dtype.storage == "pyarrow":
+        reason = (
+            "Regex pattern \"Cannot set non-string value '1' into "
+            "a StringArray.\" does not match 'Scalar must be NA or str'"
+        )
+        mark = pytest.mark.xfail(raises=AssertionError, reason=reason)
+        request.node.add_marker(mark)
+
+    arr = pd.array(["a", pd.NA], dtype=dtype)
 
     res = arr.fillna(value="b")
-    expected = pd.array(["a", "b"], dtype="string")
+    expected = pd.array(["a", "b"], dtype=dtype)
     tm.assert_extension_array_equal(res, expected)
 
     res = arr.fillna(value=np.str_("b"))
-    expected = pd.array(["a", "b"], dtype="string")
+    expected = pd.array(["a", "b"], dtype=dtype)
     tm.assert_extension_array_equal(res, expected)
 
     msg = "Cannot set non-string value '1' into a StringArray."
@@ -444,7 +409,7 @@ def test_fillna_args():
         arr.fillna(value=1)
 
 
-@td.skip_if_no("pyarrow", min_version="0.15.0")
+@td.skip_if_no("pyarrow")
 def test_arrow_array(dtype):
     # protocol added in 0.15.0
     import pyarrow as pa
@@ -452,14 +417,14 @@ def test_arrow_array(dtype):
     data = pd.array(["a", "b", "c"], dtype=dtype)
     arr = pa.array(data)
     expected = pa.array(list(data), type=pa.string(), from_pandas=True)
-    if dtype == "arrow_string":
+    if dtype.storage == "pyarrow":
         expected = pa.chunked_array(expected)
 
     assert arr.equals(expected)
 
 
-@td.skip_if_no("pyarrow", min_version="0.15.1.dev")
-def test_arrow_roundtrip(dtype, dtype_object):
+@td.skip_if_no("pyarrow")
+def test_arrow_roundtrip(dtype, string_storage2):
     # roundtrip possible from arrow 1.0.0
     import pyarrow as pa
 
@@ -467,38 +432,48 @@ def test_arrow_roundtrip(dtype, dtype_object):
     df = pd.DataFrame({"a": data})
     table = pa.table(df)
     assert table.field("a").type == "string"
-    result = table.to_pandas()
-    assert isinstance(result["a"].dtype, dtype_object)
-    tm.assert_frame_equal(result, df)
+    with pd.option_context("string_storage", string_storage2):
+        result = table.to_pandas()
+    assert isinstance(result["a"].dtype, pd.StringDtype)
+    expected = df.astype(f"string[{string_storage2}]")
+    tm.assert_frame_equal(result, expected)
     # ensure the missing value is represented by NA and not np.nan or None
     assert result.loc[2, "a"] is pd.NA
 
 
-def test_value_counts_na(dtype, request):
-    if dtype == "arrow_string":
-        reason = "TypeError: boolean value of NA is ambiguous"
-        mark = pytest.mark.xfail(reason=reason)
-        request.node.add_marker(mark)
+@td.skip_if_no("pyarrow")
+def test_arrow_load_from_zero_chunks(dtype, string_storage2):
+    # GH-41040
+    import pyarrow as pa
 
+    data = pd.array([], dtype=dtype)
+    df = pd.DataFrame({"a": data})
+    table = pa.table(df)
+    assert table.field("a").type == "string"
+    # Instantiate the same table with no chunks at all
+    table = pa.table([pa.chunked_array([], type=pa.string())], schema=table.schema)
+    with pd.option_context("string_storage", string_storage2):
+        result = table.to_pandas()
+    assert isinstance(result["a"].dtype, pd.StringDtype)
+    expected = df.astype(f"string[{string_storage2}]")
+    tm.assert_frame_equal(result, expected)
+
+
+def test_value_counts_na(dtype):
     arr = pd.array(["a", "b", "a", pd.NA], dtype=dtype)
     result = arr.value_counts(dropna=False)
-    expected = pd.Series([2, 1, 1], index=["a", pd.NA, "b"], dtype="Int64")
+    expected = pd.Series([2, 1, 1], index=arr[[0, 1, 3]], dtype="Int64")
     tm.assert_series_equal(result, expected)
 
     result = arr.value_counts(dropna=True)
-    expected = pd.Series([2, 1], index=["a", "b"], dtype="Int64")
+    expected = pd.Series([2, 1], index=arr[:2], dtype="Int64")
     tm.assert_series_equal(result, expected)
 
 
-def test_value_counts_with_normalize(dtype, request):
-    if dtype == "arrow_string":
-        reason = "TypeError: boolean value of NA is ambiguous"
-        mark = pytest.mark.xfail(reason=reason)
-        request.node.add_marker(mark)
-
-    s = pd.Series(["a", "b", "a", pd.NA], dtype=dtype)
-    result = s.value_counts(normalize=True)
-    expected = pd.Series([2, 1], index=["a", "b"], dtype="Float64") / 3
+def test_value_counts_with_normalize(dtype):
+    ser = pd.Series(["a", "b", "a", pd.NA], dtype=dtype)
+    result = ser.value_counts(normalize=True)
+    expected = pd.Series([2, 1], index=ser[:2], dtype="Float64") / 3
     tm.assert_series_equal(result, expected)
 
 
@@ -525,11 +500,11 @@ def test_use_inf_as_na(values, expected, dtype):
         tm.assert_frame_equal(result, expected)
 
 
-def test_memory_usage(dtype, request):
+def test_memory_usage(dtype):
     # GH 33963
 
-    if dtype == "arrow_string":
-        pytest.skip("not applicable")
+    if dtype.storage == "pyarrow":
+        pytest.skip(f"not applicable for {dtype.storage}")
 
     series = pd.Series(["a", "b", "c"], dtype=dtype)
 
@@ -539,8 +514,8 @@ def test_memory_usage(dtype, request):
 @pytest.mark.parametrize("float_dtype", [np.float16, np.float32, np.float64])
 def test_astype_from_float_dtype(float_dtype, dtype):
     # https://github.com/pandas-dev/pandas/issues/36451
-    s = pd.Series([0.1], dtype=float_dtype)
-    result = s.astype(dtype)
+    ser = pd.Series([0.1], dtype=float_dtype)
+    result = ser.astype(dtype)
     expected = pd.Series(["0.1"], dtype=dtype)
     tm.assert_series_equal(result, expected)
 
@@ -558,3 +533,43 @@ def test_to_numpy_na_value(dtype, nulls_fixture):
     result = arr.to_numpy(na_value=na_value)
     expected = np.array(["a", na_value, "b"], dtype=object)
     tm.assert_numpy_array_equal(result, expected)
+
+
+def test_isin(dtype, request, fixed_now_ts):
+    s = pd.Series(["a", "b", None], dtype=dtype)
+
+    result = s.isin(["a", "c"])
+    expected = pd.Series([True, False, False])
+    tm.assert_series_equal(result, expected)
+
+    result = s.isin(["a", pd.NA])
+    expected = pd.Series([True, False, True])
+    tm.assert_series_equal(result, expected)
+
+    result = s.isin([])
+    expected = pd.Series([False, False, False])
+    tm.assert_series_equal(result, expected)
+
+    result = s.isin(["a", fixed_now_ts])
+    expected = pd.Series([True, False, False])
+    tm.assert_series_equal(result, expected)
+
+
+def test_setitem_scalar_with_mask_validation(dtype):
+    # https://github.com/pandas-dev/pandas/issues/47628
+    # setting None with a boolean mask (through _putmaks) should still result
+    # in pd.NA values in the underlying array
+    ser = pd.Series(["a", "b", "c"], dtype=dtype)
+    mask = np.array([False, True, False])
+
+    ser[mask] = None
+    assert ser.array[1] is pd.NA
+
+    # for other non-string we should also raise an error
+    ser = pd.Series(["a", "b", "c"], dtype=dtype)
+    if type(ser.array) is pd.arrays.StringArray:
+        msg = "Cannot set non-string value"
+    else:
+        msg = "Scalar must be NA or str"
+    with pytest.raises(ValueError, match=msg):
+        ser[mask] = 1
